#!/usr/bin/env python

# Copyright (c) 2002 Trent Mick
# 
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""Test p4lib.py's interface to 'p4 label'."""

import os
import sys
import unittest
import pprint

import testsupport
from p4lib import P4, P4LibError


class LabelTestCase(unittest.TestCase):
    #TODO:
    # - test expected error from trying to change a locked label from a
    #   user that is not the owner
    # - have the super user change a locked label with and without force
    def test_get_label(self):
        top = os.getcwd()
        andrew = testsupport.users["andrew"]
        p4 = P4()

        try:
            os.chdir(andrew["home"])

            name = "test_get_label"
            labelDict = {
                "label": name,
                "description": name,
                "view": "//depot/...",
            }
            p4.label(label=labelDict)
            label = p4.label("test_get_label")
            self.failUnless(label["label"] == name)
            self.failUnless(label.has_key("access"))
            self.failUnless(label.has_key("description"))
            self.failUnless(label.has_key("options"))
            self.failUnless(label.has_key("owner"))
            self.failUnless(label.has_key("update"))
            self.failUnless(label.has_key("view"))
        finally:
            os.chdir(top)

    def test_create_label(self):
        top = os.getcwd()
        andrew = testsupport.users["andrew"]
        p4 = P4()

        try:
            os.chdir(andrew["home"])

            name = "test_create_label"
            labelDict = {
                "label": name,
                "description": name,
                "view": "//depot/...",
            }
            p4.label(label=labelDict)
            labels = p4.labels()
            for label in labels:
                if label["label"] == name:
                    break
            else:
                self.fail("Label '%s' not successfully created" % name)
        finally:
            os.chdir(top)

    def test_delete_label(self):
        top = os.getcwd()
        andrew = testsupport.users["andrew"]
        p4 = P4()

        try:
            os.chdir(andrew["home"])

            name = "test_delete_label"
            labelDict = {
                "label": name,
                "description": name,
                "view": "//depot/...",
            }
            p4.label(label=labelDict)
            p4.label(name=name, delete=1)

            labels = p4.labels()
            for label in labels:
                if label["label"] == name:
                    self.fail("Label '%s' was not successfully deleted" % name)
        finally:
            os.chdir(top)

    def test_update_label(self):
        top = os.getcwd()
        andrew = testsupport.users["andrew"]
        p4 = P4()

        try:
            os.chdir(andrew["home"])

            name = "test_update_label"
            descBefore = "before"
            descAfter = "after"

            # Create a label and then update it.
            labelDict = {
                "label": name,
                "description": descBefore,
                "view": "//depot/...",
            }
            p4.label(label=labelDict)
            p4.label(name=name, label={"description": descAfter})

            label = p4.label(name=name)
            self.failUnless(label["description"] == descAfter)
        finally:
            os.chdir(top)

    def test_update_label_no_change(self):
        top = os.getcwd()
        andrew = testsupport.users["andrew"]
        p4 = P4()

        try:
            os.chdir(andrew["home"])

            name = "test_update_label_no_change"

            # Create a label and then update it.
            labelDict = {
                "label": name,
                "description": name,
                "view": "//depot/...",
            }
            p4.label(label=labelDict)
            p4.label(name=name, label={})
            # Update the client.
            result = p4.label(name=name, label={})
            self.failUnless(result["label"] == name)
            self.failUnless(result["action"] == "not changed")

        finally:
            os.chdir(top)


def suite():
    """Return a unittest.TestSuite to be used by test.py."""
    return unittest.makeSuite(LabelTestCase)

